﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SimpleCalculator
{
    public class CalcModel : SimpleCalculator.ICalcModel
    {
        /// <summary>Dictionary for easy display of operators</summary>
        private Dictionary<CalcEnums.Operators, string> opDic = new Dictionary<CalcEnums.Operators, string>() {
            { CalcEnums.Operators.None, "" },
            { CalcEnums.Operators.Add, "+" },
            { CalcEnums.Operators.Subtract, "-" },
            { CalcEnums.Operators.Multiply, "*" },
            { CalcEnums.Operators.Divide, "/" }
        };

        public double? _num1;
        public double? _num2;
        /// <summary>1st number in our equation</summary>
        public double? Num1
        {
            get { return _num1; }
        }
        /// <summary>2nd number in our equation</summary>
        public double? Num2
        {
            get { return _num2; }
        }
        /// <summary>Operator in our equation</summary>
        public CalcEnums.Operators Operator { get; set; }

        /// <summary>This calc model uses doubles for all numbers.
        /// Since numbers are hand-entered and are limited to only 2 numbers, none
        /// of the weirdness of floating point numbers should arise
        /// (such as using != 0 as opposed to a range check)</summary>
        public CalcModel()
        {
            Reset();
        }

        /// <summary>set number 1</summary>
        /// <param name="num">number to set</param>
        public void SetNum1(double num)
        {
            _num1 = num;
        }
        /// <summary>set number 2</summary>
        /// <param name="num">number to set</param>
        public void SetNum2(double num)
        {
            _num2 = num;
        }

        bool num1dec = false;
        bool num2dec = false;
        /// <summary>append a decimal point on number 1. all additional numbers will follow dec point</summary>
        public void AppendNum1DecimalPoint()
        {
            num1dec = true;
        }
        /// <summary>append a decimal point on number 2. all additional numbers will follow dec point</summary>
        public void AppendNum2DecimalPoint()
        {
            num2dec = true;
        }

        /// <summary>append a digit on the end of number 1</summary>
        /// <param name="digit">digit to append</param>
        public void AppendNum1Digit(int digit)
        {
            if (Num1 == null)
            {
                _num1 = digit;
                return;
            }
            _num1 = appendDigit(digit, (double)Num1, num1dec);
            num1dec = false;
        }
        /// <summary>append a digit on the end of number 2</summary>
        /// <param name="digit">digit to append</param>
        public void AppendNum2Digit(int digit)
        {
            if (Num2 == null)
            {
                _num2 = digit;
                return;
            }
            _num2 = appendDigit(digit, (double)Num2, num2dec);
            num2dec = false;
        }
        /// <summary>Append digit 0-9</summary>
        /// <param name="digit">digit to append</param>
        /// <param name="dest">number to append to</param>
        /// <param name="decFlag">whether to add a decimal point</param>
        /// <returns>new double with digit on the end</returns>
        private double appendDigit(int digit, double dest, bool decFlag)
        {
            // only accept the 1st digit. ignore sign. it's only supposed to be a number 0-9 anyway
            string digitStr = digit.ToString().Trim('-')[0].ToString();
            string destStr = dest.ToString();

            // handle scientific notation
            int eIndex = destStr.IndexOf('E');
            string exponent = "";
            if (eIndex > -1)
            {
                destStr = destStr.Substring(0, eIndex);
                exponent = destStr.Substring(eIndex);
            }

            // handle decimal point
            if (decFlag && destStr.IndexOf('.') == -1)
                destStr += ".";

            return Double.Parse(destStr + digit + exponent);
        }

        /// <summary>Reset numbers and operator</summary>
        public void Reset()
        {
            _num1 = null;
            _num2 = null;
            num1dec = false;
            num2dec = false;
            Operator = CalcEnums.Operators.None;
        }

        /// <summary>Retrieve the result of the Num1 Op Num2 calculation</summary>
        /// <returns>Calculation result. Will throw exceptions for incomplete input.</returns>
        public double GetResult()
        {
            if (Num1 == null)
                throw new Exception("Number 1 has not been set");
            if (Num2 == null)
                throw new Exception("Number 2 has not been set");
            if (Operator == CalcEnums.Operators.None)
                throw new Exception("Calculator has no operator");

            // cast to non-nullable
            double num1 = (double)Num1;
            double num2 = (double)Num2;

            switch (Operator)
            {
                case CalcEnums.Operators.Add:
                    return num1 + num2;
                case CalcEnums.Operators.Subtract:
                    return num1 - num2;
                case CalcEnums.Operators.Multiply:
                    return num1 * num2;
                case CalcEnums.Operators.Divide:
                default:
                    if (num2 == 0)
                        throw new Exception("Divide by 0");
                    return num1 / num2;
            }
        }

        /// <summary>Get a string representation of the formula</summary>
        /// <returns></returns>
        public string GetFormula()
        {
            string num1 = Num1 == null ? "" : Num1.ToString();
            string num2 = Num2 == null ? "" : Num2.ToString();

            return num1 + opDic[Operator] + num2;
        }
    }
}
